package hikapro.com.backpack.model;


import android.os.Message;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Hashtable;
import java.util.List;

import hikapro.com.backpack.model.dao.Command;
import hikapro.com.backpack.model.dao.DAO;
import hikapro.com.backpack.model.dao.Event;
import hikapro.com.backpack.model.entities.Category;
import hikapro.com.backpack.model.entities.Item;
import hikapro.com.backpack.presenter.Presenter;

/**
 * Created by tariel on 22/04/16.
 */
public class ItemModel implements Model.Item {

    private Presenter.ItemList  presenter;
    private List<Category>      rawCategories;
    private List<Category>      sortedCategories;
    private List<Item>          rawItems;

    private DAO                 dao;
    private int                 currentSet;
    private Hashtable<Integer, Category>      categoriesCache;
    private List<Item>          itemsCache;
    private List<Item>          itemsDiscardCache;

    private Hashtable<Category, List<Item>> items;
    private Hashtable<Integer, List<Item>> cache;

    public ItemModel() {
        this.rawCategories      = new ArrayList<>();
        this.rawItems           = new ArrayList<>();
        this.sortedCategories   = new ArrayList<>();

        this.categoriesCache    = new Hashtable<>(20, 0.9f);

        this.itemsCache         = new ArrayList<>();
        this.itemsDiscardCache  = new ArrayList<>();
        this.cache              = new Hashtable<>(12, 0.9f);
        this.dao                = DAO.getInstance();
        dao.registerObserver(this);
    }

    // categories -->

    @Override
    public Category getCategoryByPosition(int position) {
        Category ret = null;
        if (cache.containsKey(currentSet))
            ret = categoriesCache.get(cache.get(currentSet).get(position).getCategory());
        return ret;

    }
    @Override
    public int getCategoriesCount() {
        return categoriesCache.size();
    }

    // categories <--

    // items -->

    @Override
    public int insertItem(Item item) {
        List<Item> iList = cache.get(currentSet);
        iList.add(item);
        Collections.sort(iList);
        cache.put(currentSet, iList);
        // TODO write to database
        return 0;
    }
    @Override
    public boolean deleteItem(Item item) {
        if (isPendingRemoval(item))
            itemsDiscardCache.remove(item);
        cache.get(currentSet).remove(item);
        //TODO write to ds

        return false;
    }

    @Override
    public Item findItem(int id) {
        Item item = null;
        for (Item i : rawItems) {
            if (i.getId() == id) {
                item = i;
                break;
            }
        }
        return item;
    }
    @Override
    public Item getItemByPosition(int position) {
        Item ret = null;
        if (cache.containsKey(currentSet))
            ret = cache.get(currentSet).get(position);
        return ret;
    }


    @Override
    public void filter(String query) {

        if (query.isEmpty()) {
            Message command = Message.obtain();
            command.what = Command.SET_GET_ITEMS;
            command.arg1 = presenter.getCurrentSet().getId();
            dao.executeCommand(command);
        } else {
            query = query.toLowerCase();
            String name;
            List<Item> newList = new ArrayList<>(20);
            List<Item> oldList = cache.get(currentSet);
            for (Item item : oldList) {
                name = item.getName().toLowerCase();
                if (name.contains(query)) {
                    newList.add(item);
                }
            }
            cache.put(currentSet, newList);
        }
    }

    @Override
    public int getHeaderId(int position) {
        return cache.containsKey(currentSet) ? cache.get(currentSet).get(position).getCategory() : -1;
    }

    @Override
    public int getItemId(int position) {
        return cache.containsKey(currentSet) ? cache.get(currentSet).get(position).getId() : -1;
    }

    @Override
    public void clear() {
        if (cache.containsKey(currentSet))
            cache.get(currentSet).clear();
    }

    @Override
    public boolean isPendingRemoval(Item item) {
        return itemsDiscardCache.contains(item);
    }

    @Override
    public void pendingRemoveCancel(Item item) {
        if (itemsDiscardCache.contains(item))
            itemsDiscardCache.remove(item);
    }

    @Override
    public void pendingRemove(Item item) {
        itemsDiscardCache.add(item);
    }

    @Override
    public int getItemsCount() {
        boolean is = cache.containsKey(currentSet);
        return is ? cache.get(currentSet).size() : 0;
    }

    // items <--

    // events -->

    @Override
    public void notifyDataSetChanged() {
        presenter.notifyDataSetChanged();
    }

    @Override
    public void onDestroy(boolean isConfigurationChanging) {
        if ( !isConfigurationChanging ) {
            presenter = null;
        }
    }

    private void sendMessage(String message) {
        presenter.showMessage(message);
    }
    // events <--

    // process -->

    @Override
    public void executeQuery() {
        Message command;

        if (cache.contains(currentSet)) {
            notifyDataSetChanged();
        } else {
            if (categoriesCache.isEmpty()) {
                command = Message.obtain();
                command.what = Command.ITEM_GET_CATEGORIES;
                dao.executeCommand(command);
            }
            command = Message.obtain();
            command.what = Command.SET_GET_ITEMS;
            command.arg1 = presenter.getCurrentSet().getId();
            dao.executeCommand(command);

        }
    }
    @Override
    public void onEvent(Message event) {

        switch (event.what) {
            case Event.SET_ITEMS_LOAD_ERROR :
                break;
            case Event.ITEM_FROM_SET_ERROR :
                break;
            case Event.ITEM_DELETE_ERROR :
                break;
            case Event.ITEM_PACK_ERROR :
                break;
            case Event.ITEM_UNPACK_ERROR :
                break;
            case Event.ITEM_INSERT_ERROR :
                break;
            case Event.ITEM_CATEGORY_LOAD_ERROR :
                break;
            case Event.SET_ITEMS_LOAD_COMPLETED :
                List<Item> res = (List<Item>) event.obj;
                cache.put(event.arg1, res);
                notifyDataSetChanged();
                break;
            case Event.ITEM_CATEGORY_LOAD_COMPLETED :
                categoriesCache = (Hashtable<Integer, Category>)event.obj;
                break;
            case Event.ITEM_FROM_SET_DELETED :
                break;
            case Event.ITEM_DELETED :
                break;
            case Event.ITEM_PACKED :
                break;
            case Event.ITEM_UNPACKED :
                break;
            case Event.ITEM_INSERTED :
                break;
        }
    }

    // process <--

    // other -->


    @Override
    public void setPresenter(Presenter.ItemList presenter) {
        this.presenter = presenter;
        this.currentSet = presenter.getCurrentSet().getId();
    }

    @Override
    public Presenter.ItemList getPresenter() {
        return presenter;
    }

    private void initData() {
        List<Integer> ids       = presenter.getCurrentSet().getItems();
        List<Item> sortedItems  = new ArrayList<>(ids.size());
        Category category;

        for (Item item : rawItems) {
            if (ids.contains(item.getId())) {
                sortedItems.add(item);
            }
        }
        Collections.sort(sortedItems);
        items = new Hashtable<>(sortedItems.size());

        for (Item item : sortedItems) {
            category = findCategory(item.getCategory());
            if (category != null) {
                if (items.containsKey(category)) {
                    items.get(category).add(item);
                } else {
                    List<Item> innerList = new ArrayList<>(20);
                    innerList.add(item);
                    items.put(category, innerList);
                }
            }
        }
        Category[] array = items.keySet().toArray(new Category[items.size()]);
        sortedCategories = Arrays.asList(array);
        notifyDataSetChanged();
    }
    private Category findSortedCategory(int categoryId) {
        Category category = null;
        for (Category c : sortedCategories) {
            if (c.getId() == categoryId) {
                category = c;
                break;
            }
        }
        return category;
    }
    private Category findCategory(int categoryId) {
        Category category = null;

        for (Category c : rawCategories) {
            if (c.getId() == categoryId) {
                category = c;
                break;
            }
        }
        return category;
    }
    // other <--
}
