package hikapro.com.backpack.view.fragments;


import android.app.Activity;
import android.app.Fragment;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.SearchView;
import android.widget.Toast;

import hikapro.com.backpack.R;
import hikapro.com.backpack.model.entities.Item;
import hikapro.com.backpack.model.entities.Set;
import hikapro.com.backpack.presenter.ItemListPresenter;
import hikapro.com.backpack.presenter.Presenter;


public class ItemListFragment extends Fragment implements hikapro.com.backpack.view.View.ItemList,
        SearchView.OnQueryTextListener {

    private static final String BUNDLE_SET_KEY = "BUNDLE_SET_KEY";
    private hikapro.com.backpack.view.View.ActivityCallback activityCallback;
    private Presenter.ItemList  presenter;

    public ItemListFragment() {
        // Required empty public constructor
    }

    protected static ItemListFragment construct() {
        return new ItemListFragment();
    }

    public static ItemListFragment newFromSet(Set set) {
        ItemListFragment ret = ItemListFragment.construct();
        Bundle args = new Bundle();
        args.putSerializable(BUNDLE_SET_KEY, set);
        ret.setArguments(args);
        return ret;
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {

        inflater.inflate(R.menu.menu_main, menu);

        final MenuItem item = menu.findItem(R.id.action_search);
        final SearchView searchView = (SearchView) item.getActionView();
        searchView.setOnQueryTextListener(this);
        super.onCreateOptionsMenu(menu, inflater);

    }

    @Override
    public boolean onQueryTextSubmit(String query) {
        return false;
    }

    @Override
    public boolean onQueryTextChange(String newText) {
        presenter.filter(newText);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        return super.onOptionsItemSelected(item);
    }

    // life cycle -->
    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        try {
            activityCallback = (hikapro.com.backpack.view.View.ActivityCallback) context;
        } catch (ClassCastException e) {
            throw new ClassCastException(context.toString()
                    + " must implement activityCallback");
        }
    }
    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            activityCallback = (hikapro.com.backpack.view.View.ActivityCallback) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString()
                    + " must implement activityCallback");
        }
        Log.i(this.toString(), "onAttach");
    }
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
        Log.i(this.toString(), "onCreate");
    }
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = presenter.onCreateView(inflater, container, savedInstanceState);
        presenter.setView(this);
        Log.i(this.toString(), "onCreateView");
        return view;
    }
    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        Log.i(this.toString(), "onActivityCreated");
    }
    @Override
    public void onStart() {
        super.onStart();
        Log.i(this.toString(), "onStart");
    }
    @Override
    public void onResume() {
        super.onResume();
        Log.i(this.toString(), "onResume");
    }
    @Override
    public void onStop() {
        super.onStop();
        Log.i(this.toString(), "onStop");
    }
    @Override
    public void onDestroyView() {
        super.onDestroyView();
        presenter.onDestroy(true); // TODO isChangingConfigurations
        Log.i(this.toString(), "onDestroyView");
    }
    @Override
    public void onDestroy() {
        super.onDestroy();
        presenter.onDestroy(false); // TODO isChangingConfigurations
        Log.i(this.toString(), "onDestroy");
    }
    @Override
    public void onDetach() {
        super.onDetach();
        Log.i(this.toString(), "onDetach");
    }
    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        Log.i(this.toString(), "onSaveInstanceState");
    }

    // life cycle <--

    @Override
    public void showItemDetail(Item item) {
        activityCallback.startItemDetailFragment(item);
    }

    @Override
    public void setPresenter(Presenter.ItemList presenter) {
        this.presenter = presenter;
    }

    @Override
    public Context getAppContext() {
        return this.getActivity().getApplicationContext();
    }

    @Override
    public Context getActivityContext() {
        return this.getActivity();
    }

    public Set getSet() {
        return (Set) getArguments().getSerializable(BUNDLE_SET_KEY);
    }


}
